@@ -114,7 +114,7 @@ impl EncodingLevel {
 /// management; symbol-related operations (e.g. setting field IDs and annotations or writing symbol
 /// values) require a valid symbol ID to be provided by the caller.
 ///
-/// To produce a valid binary Ion stream, the writer MUST call [write_ion_version_marker] before
+/// To produce a valid binary Ion stream, the writer MUST call [Writer::write_ion_version_marker] before
 /// writing any data.
 #[derive(Debug)]
 pub struct RawBinaryWriter<W: Write> {
@@ -777,6 +777,10 @@ impl<'a, W: Write> Writer for RawBinaryWriter<W> {
             Ok(())
         })?;
 
+        // Now that we know how large the container's header is, add its length to the
+        // calculated container size.
+        let container_size = container_size + header_io_range.len();
+
         // Retrieve this container's header byte range from io_ranges
         let td_io_range = self
             .io_ranges
@@ -844,7 +848,7 @@ mod writer_tests {
 
     use super::*;
     use crate::raw_symbol_token::{local_sid_token, RawSymbolToken};
-    use crate::symbol_table::Symbol;
+    use crate::symbol::Symbol;
     use crate::StreamReader;
     use num_traits::Float;
     use std::convert::TryInto;
