@@ -7,9 +7,10 @@ use crate::constants::v1_0::{system_symbol_ids, SYSTEM_SYMBOLS};
 use crate::raw_reader::{RawReader, RawStreamItem};
 use crate::raw_symbol_token::RawSymbolToken;
 use crate::result::{decoding_error, decoding_error_raw, illegal_operation, IonError, IonResult};
-use crate::symbol_table::Symbol;
+use crate::symbol::Symbol;
 use crate::system_reader::LstPosition::*;
 use crate::types::decimal::Decimal;
+use crate::types::integer::Integer;
 use crate::types::timestamp::Timestamp;
 use crate::{IonType, RawBinaryReader, StreamReader, SymbolTable};
 
@@ -455,7 +456,7 @@ impl<R: RawReader> SystemReader<R> {
         self.raw_reader.annotations().map(|a| a.unwrap())
     }
 
-    fn symbol_table(&self) -> &SymbolTable {
+    pub fn symbol_table(&self) -> &SymbolTable {
         &self.symbol_table
     }
 
@@ -712,6 +713,7 @@ impl<R: RawReader> StreamReader for SystemReader<R> {
             fn ion_type(&self) -> Option<IonType>;
             fn read_null(&mut self) -> IonResult<IonType>;
             fn read_bool(&mut self) -> IonResult<bool>;
+            fn read_integer(&mut self) -> IonResult<Integer>;
             fn read_i64(&mut self) -> IonResult<i64>;
             fn read_f32(&mut self) -> IonResult<f32>;
             fn read_f64(&mut self) -> IonResult<f64>;
