@@ -1,8 +1,9 @@
 mod parent_container;
+pub(crate) mod parse_result;
 pub(in crate::text) mod parsers;
 pub mod raw_text_reader;
 pub mod raw_text_writer;
 mod text_buffer;
 mod text_data_source;
 mod text_value;
-mod text_writer;
+pub(crate) mod text_writer;
