@@ -1,37 +1,49 @@
 //! Parsing logic for the text representation of blob values.
 
-use base64::DecodeError;
+use crate::text::parse_result::{fatal_parse_error, IonParseResult, UpgradeIResult};
 use nom::branch::alt;
 use nom::bytes::streaming::{is_a, tag};
-use nom::character::streaming::{alphanumeric1, char};
-use nom::combinator::{map_res, opt, recognize};
-use nom::multi::many0_count;
-use nom::sequence::{delimited, pair};
-use nom::IResult;
+use nom::character::streaming::alphanumeric1;
+use nom::combinator::{opt, recognize};
+use nom::multi::{many0_count, many1_count};
+use nom::sequence::{delimited, pair, terminated};
 
-use crate::text::parsers::whitespace;
+use crate::text::parsers::{whitespace, WHITESPACE_CHARACTERS};
 use crate::text::text_value::TextValue;
 
 /// Matches the text representation of a blob value, decodes it, and returns the resulting bytes
 /// as a [TextValue::Blob].
-pub(crate) fn parse_blob(input: &str) -> IResult<&str, TextValue> {
-    map_res::<_, _, _, _, DecodeError, _, _>(
-        delimited(
-            pair(tag("{{"), opt(whitespace)),
+pub(crate) fn parse_blob(input: &str) -> IonParseResult<TextValue> {
+    let (remaining_input, base64_text) = delimited(
+        pair(tag("{{"), opt(whitespace)),
+        // Whitespace can appear in between chunks of the base64 data. Here we match on any
+        // number of (base64_data, optional_whitespace) chunks and return the &str that contained them.
+        recognize(many0_count(terminated(
             recognize_base64_data,
-            pair(opt(whitespace), tag("}}")),
-        ),
-        |base64_text| Ok(TextValue::Blob(base64::decode(base64_text)?)),
-    )(input)
+            opt(whitespace),
+        ))),
+        pair(opt(whitespace), tag("}}")),
+    )(input)?;
+
+    let decode_result = if base64_text.contains(WHITESPACE_CHARACTERS) {
+        let sanitized = base64_text.replace(WHITESPACE_CHARACTERS, "");
+        base64::decode(sanitized)
+    } else {
+        base64::decode(base64_text)
+    };
+
+    // If the parser matched the input text ("{{ ... }}"), it's definitely supposed to be
+    // a blob. If we can't decode that text as base64, the stream is malformed.
+    match decode_result {
+        Ok(data) => Ok((remaining_input, TextValue::Blob(data))),
+        Err(e) => fatal_parse_error(base64_text, format!("could not decode base64 data: {}", e)),
+    }
 }
 
-/// Matches a series of valid base64-encoded characters. (This function does not attempt to decode
-/// the matched value.)
-fn recognize_base64_data(input: &str) -> IResult<&str, &str> {
-    recognize(pair(
-        many0_count(alt((alphanumeric1, is_a("+/")))),
-        pair(opt(char('=')), opt(char('='))),
-    ))(input)
+/// Matches a series of valid base64-encoded characters. This function does not attempt to decode
+/// the matched value; it is possible for it to match a string that cannot be decoded successfully.
+fn recognize_base64_data(input: &str) -> IonParseResult<&str> {
+    recognize(many1_count(alt((alphanumeric1, is_a("+/=")))))(input).upgrade()
 }
 
 #[cfg(test)]
