@@ -4,21 +4,23 @@ use std::str::FromStr;
 use nom::branch::alt;
 use nom::bytes::streaming::tag;
 use nom::character::complete::digit1;
-use nom::character::streaming::{char, one_of};
-use nom::combinator::{map, map_res, opt, recognize};
+use nom::character::streaming::{char, one_of, satisfy};
+use nom::combinator::{map, opt, recognize};
 use nom::sequence::{pair, preceded, separated_pair, terminated, tuple};
 use nom::IResult;
 use num_bigint::BigUint;
 
-use crate::result::IonError;
-use crate::text::parsers::{digit, stop_character, trim_zeros_expect_i32, trim_zeros_expect_u32};
+use crate::text::parse_result::{
+    fatal_parse_error, IonParseResult, OrFatalParseError, UpgradeIResult,
+};
+use crate::text::parsers::{stop_character, trim_zeros_and_parse_i32, trim_zeros_and_parse_u32};
 use crate::text::text_value::TextValue;
 use crate::types::decimal::Decimal;
 use crate::types::timestamp::{FractionalSecondSetter, Timestamp};
 
 /// Matches the text representation of a timestamp value and returns the resulting Timestamp
 /// as a [TextValue::Timestamp].
-pub(crate) fn parse_timestamp(input: &str) -> IResult<&str, TextValue> {
+pub(crate) fn parse_timestamp(input: &str) -> IonParseResult<TextValue> {
     alt((
         timestamp_precision_y,
         timestamp_precision_ym,
@@ -31,93 +33,88 @@ pub(crate) fn parse_timestamp(input: &str) -> IResult<&str, TextValue> {
 
 /// Matches the text representation of a timestamp value with year precision (e.g. `2018T`) and
 /// returns the resulting Timestamp as a [TextValue::Timestamp].
-fn timestamp_precision_y(input: &str) -> IResult<&str, TextValue> {
-    map_res::<_, _, _, _, IonError, _, _>(
-        terminated(year, pair(tag("T"), stop_character)),
-        |year| {
-            let timestamp = Timestamp::with_year(year).build()?;
-            Ok(TextValue::Timestamp(timestamp))
-        },
-    )(input)
+fn timestamp_precision_y(input: &str) -> IonParseResult<TextValue> {
+    let (remaining, year) = terminated(year, pair(tag("T"), stop_character))(input)?;
+    let timestamp = Timestamp::with_year(year)
+        .build()
+        .or_fatal_parse_error(input, "could not create timestamp")?
+        .1;
+    Ok((remaining, TextValue::Timestamp(timestamp)))
 }
 
 /// Matches the text representation of a timestamp value with month precision (e.g. `2018-06T`) and
 /// returns the resulting Timestamp as a [TextValue::Timestamp].
-fn timestamp_precision_ym(input: &str) -> IResult<&str, TextValue> {
-    map_res::<_, _, _, _, IonError, _, _>(
-        terminated(pair(year, month), pair(tag("T"), stop_character)),
-        |(year, month)| {
-            let timestamp = Timestamp::with_year(year).with_month(month).build()?;
-            Ok(TextValue::Timestamp(timestamp))
-        },
-    )(input)
+fn timestamp_precision_ym(input: &str) -> IonParseResult<TextValue> {
+    let (remaining, (year, month)) =
+        terminated(pair(year, month), pair(tag("T"), stop_character))(input)?;
+    let timestamp = Timestamp::with_year(year)
+        .with_month(month)
+        .build()
+        .or_fatal_parse_error(input, "could not create timestamp")?
+        .1;
+    Ok((remaining, TextValue::Timestamp(timestamp)))
 }
 
 /// Matches the text representation of a timestamp value with day precision (e.g. `2018-06-12T`) and
 /// returns the resulting Timestamp as a [TextValue::Timestamp].
-fn timestamp_precision_ymd(input: &str) -> IResult<&str, TextValue> {
-    map_res::<_, _, _, _, IonError, _, _>(
-        terminated(
-            tuple((year, month, day)),
-            pair(opt(tag("T")), stop_character),
-        ),
-        |(year, month, day)| {
-            let timestamp = Timestamp::with_ymd(year, month, day).build()?;
-            Ok(TextValue::Timestamp(timestamp))
-        },
-    )(input)
+fn timestamp_precision_ymd(input: &str) -> IonParseResult<TextValue> {
+    let (remaining, (year, month, day)) = terminated(
+        tuple((year, month, day)),
+        pair(opt(tag("T")), stop_character),
+    )(input)?;
+    let timestamp = Timestamp::with_ymd(year, month, day)
+        .build()
+        .or_fatal_parse_error(input, "could not create timestamp")?
+        .1;
+    Ok((remaining, TextValue::Timestamp(timestamp)))
 }
 
 /// Matches the text representation of a timestamp value with minute precision
 /// (e.g. `2018-06-12T:23:57-05:00`) and returns the resulting Timestamp as
 /// a [TextValue::Timestamp].
-fn timestamp_precision_ymd_hm(input: &str) -> IResult<&str, TextValue> {
-    map_res::<_, _, _, _, IonError, _, _>(
-        terminated(
-            pair(tuple((year, month, day, hour_and_minute)), timezone_offset),
-            stop_character,
-        ),
-        |((year, month, day, (hour, minute)), offset)| {
-            let builder = Timestamp::with_ymd(year, month, day).with_hour_and_minute(hour, minute);
-            let timestamp = if let Some(minutes) = offset {
-                builder.build_at_offset(minutes)
-            } else {
-                builder.build_at_unknown_offset()
-            }?;
-            Ok(TextValue::Timestamp(timestamp))
-        },
-    )(input)
+fn timestamp_precision_ymd_hm(input: &str) -> IonParseResult<TextValue> {
+    let (remaining, ((year, month, day, (hour, minute)), offset)) = terminated(
+        pair(tuple((year, month, day, hour_and_minute)), timezone_offset),
+        stop_character,
+    )(input)?;
+    let builder = Timestamp::with_ymd(year, month, day).with_hour_and_minute(hour, minute);
+    let timestamp = if let Some(minutes) = offset {
+        builder.build_at_offset(minutes)
+    } else {
+        builder.build_at_unknown_offset()
+    }
+    .or_fatal_parse_error(input, "could not create timestamp")?
+    .1;
+    Ok((remaining, TextValue::Timestamp(timestamp)))
 }
 
 /// Matches the text representation of a timestamp value with second precision
 /// (e.g. `2018-06-12T:23:57:45-05:00`) and returns the resulting Timestamp as
 /// a [TextValue::Timestamp].
-fn timestamp_precision_ymd_hms(input: &str) -> IResult<&str, TextValue> {
-    map_res::<_, _, _, _, IonError, _, _>(
-        terminated(
-            pair(
-                tuple((year, month, day, hour_and_minute, second)),
-                timezone_offset,
-            ),
-            stop_character,
+fn timestamp_precision_ymd_hms(input: &str) -> IonParseResult<TextValue> {
+    let (remaining, ((year, month, day, (hour, minute), second), offset)) = terminated(
+        pair(
+            tuple((year, month, day, hour_and_minute, second)),
+            timezone_offset,
         ),
-        |((year, month, day, (hour, minute), second), offset)| {
-            let builder = Timestamp::with_ymd(year, month, day).with_hms(hour, minute, second);
-            let timestamp = if let Some(minutes) = offset {
-                builder.build_at_offset(minutes)
-            } else {
-                builder.build_at_unknown_offset()
-            }?;
-            Ok(TextValue::Timestamp(timestamp))
-        },
-    )(input)
+        stop_character,
+    )(input)?;
+    let builder = Timestamp::with_ymd(year, month, day).with_hms(hour, minute, second);
+    let timestamp = if let Some(minutes) = offset {
+        builder.build_at_offset(minutes)
+    } else {
+        builder.build_at_unknown_offset()
+    }
+    .or_fatal_parse_error(input, "could not create timestamp")?
+    .1;
+    Ok((remaining, TextValue::Timestamp(timestamp)))
 }
 
 /// Matches the text representation of a timestamp value with fractional second precision
 /// (e.g. `2018-06-12T:23:57:45.993-05:00`) and returns the resulting Timestamp as
 /// a [TextValue::Timestamp].
-fn timestamp_precision_ymd_hms_fractional(input: &str) -> IResult<&str, TextValue> {
-    map_res::<_, _, _, _, IonError, _, _>(
+fn timestamp_precision_ymd_hms_fractional(input: &str) -> IonParseResult<TextValue> {
+    let (remaining, ((year, month, day, (hour, minute), second, fractional_text), offset)) =
         terminated(
             pair(
                 tuple((
@@ -131,38 +128,41 @@ fn timestamp_precision_ymd_hms_fractional(input: &str) -> IResult<&str, TextValu
                 timezone_offset,
             ),
             stop_character,
-        ),
-        |((year, month, day, (hour, minute), second, fractional), offset)| {
-            let builder = Timestamp::with_ymd(year, month, day).with_hms(hour, minute, second);
-            let builder = assign_fractional_seconds(fractional, builder);
-            let timestamp = if let Some(minutes) = offset {
-                builder.build_at_offset(minutes)?
-            } else {
-                builder.build_at_unknown_offset()?
-            };
-            Ok(TextValue::Timestamp(timestamp))
-        },
-    )(input)
+        )(input)?;
+    let builder = Timestamp::with_ymd(year, month, day).with_hms(hour, minute, second);
+    let (_, builder) = assign_fractional_seconds(fractional_text, builder)?;
+    let timestamp = if let Some(minutes) = offset {
+        builder.build_at_offset(minutes)
+    } else {
+        builder.build_at_unknown_offset()
+    }
+    .or_fatal_parse_error(input, "could not create timestamp")?
+    .1;
+    Ok((remaining, TextValue::Timestamp(timestamp)))
 }
 
 /// Parses the fractional seconds and stores it in the [FractionalSecondSetter].
 fn assign_fractional_seconds(
-    fractional: &str,
+    fractional_text: &str,
     mut setter: FractionalSecondSetter,
-) -> FractionalSecondSetter {
-    let number_of_digits = fractional.len();
+) -> IonParseResult<FractionalSecondSetter> {
+    let number_of_digits = fractional_text.len();
     // If the precision is less than or equal to nanoseconds...
     if number_of_digits <= 9 {
         // Convert the number to nanoseconds and make a note of its original precision.
         let power = 9 - number_of_digits;
-        let nanoseconds =
-            trim_zeros_expect_u32(fractional, "fractional seconds") * 10u32.pow(power as u32);
+        let (_, fractional) = trim_zeros_and_parse_u32(fractional_text, "fractional seconds")?;
+        let nanoseconds = fractional * 10u32.pow(power as u32);
         setter = setter.with_nanoseconds_and_precision(nanoseconds, number_of_digits as u32);
     } else {
         // Otherwise, the number's precision is great enough that we'll need to construct a Decimal
         // to store it without loss of fidelity.
-        let coefficient =
-            BigUint::from_str(fractional).expect("parsing fractional seconds as BigUint failed");
+        let coefficient = BigUint::from_str(fractional_text)
+            .or_fatal_parse_error(
+                fractional_text,
+                "parsing fractional seconds as BigUInt failed",
+            )?
+            .1;
         let mut digit_count = 1i64;
         let mut tmp_coefficient = coefficient.clone();
         let ten = BigUint::from(10u32);
@@ -173,92 +173,88 @@ fn assign_fractional_seconds(
         let decimal = Decimal::new(coefficient, -digit_count);
         setter = setter.with_fractional_seconds(decimal);
     }
-    setter
+    Ok(("", setter))
 }
 
 /// Matches a four-digit year, returning it as a u32.
-fn year(input: &str) -> IResult<&str, u32> {
+fn year(input: &str) -> IonParseResult<u32> {
     let y = digit;
-    map(recognize(tuple((y, y, y, y))), |year| {
-        trim_zeros_expect_u32(year, "year")
-    })(input)
+    let (remaining, year_text) = recognize(tuple((y, y, y, y)))(input).upgrade()?;
+    let (_, year) = trim_zeros_and_parse_u32(year_text, "year")?;
+    Ok((remaining, year))
 }
 
 /// Matches a two-digit month, returning it as a u32.
-fn month(input: &str) -> IResult<&str, u32> {
-    map(
-        preceded(
-            tag("-"),
-            recognize(alt((
-                pair(char('0'), one_of("123456789")),
-                pair(char('1'), one_of("012")),
-            ))),
-        ),
-        |month: &str| trim_zeros_expect_u32(month, "month"),
+fn month(input: &str) -> IonParseResult<u32> {
+    let (remaining, month_text) = preceded(
+        tag("-"),
+        recognize(alt((
+            pair(char('0'), one_of("123456789")),
+            pair(char('1'), one_of("012")),
+        ))),
     )(input)
+    .upgrade()?;
+    let (_, month) = trim_zeros_and_parse_u32(month_text, "month")?;
+    Ok((remaining, month))
 }
 
 /// Matches a two-digit day, returning it as a u32.
-fn day(input: &str) -> IResult<&str, u32> {
-    map(
-        preceded(
-            tag("-"),
-            recognize(alt((
-                pair(char('0'), one_of("123456789")),
-                pair(one_of("12"), digit),
-                pair(char('3'), one_of("01")),
-            ))),
-        ),
-        |day| trim_zeros_expect_u32(day, "day"),
+fn day(input: &str) -> IonParseResult<u32> {
+    let (remaining, day_text) = preceded(
+        tag("-"),
+        recognize(alt((
+            pair(char('0'), one_of("123456789")),
+            pair(one_of("12"), digit),
+            pair(char('3'), one_of("01")),
+        ))),
     )(input)
+    .upgrade()?;
+
+    let (_, day) = trim_zeros_and_parse_u32(day_text, "day")?;
+    Ok((remaining, day))
 }
 
 /// Matches a 'T' followed by the hour and minute (`T15:35`), returning them as as a (u32, u32) pair.
-fn hour_and_minute(input: &str) -> IResult<&str, (u32, u32)> {
-    map(
-        preceded(
-            tag("T"),
-            separated_pair(
-                recognize(alt((
-                    pair(one_of("01"), digit),
-                    pair(one_of("2"), one_of("0123")),
-                ))),
-                tag(":"),
-                recognize(pair(one_of("012345"), digit)),
-            ),
+fn hour_and_minute(input: &str) -> IonParseResult<(u32, u32)> {
+    let (remaining, (hours_text, minutes_text)) = preceded(
+        tag("T"),
+        separated_pair(
+            recognize(alt((
+                pair(one_of("01"), digit),
+                pair(one_of("2"), one_of("0123")),
+            ))),
+            tag(":"),
+            recognize(pair(one_of("012345"), digit)),
         ),
-        |(hours, minutes)| {
-            let hours = trim_zeros_expect_u32(hours, "hours");
-            let minutes = trim_zeros_expect_u32(minutes, "minutes");
-            (hours, minutes)
-        },
     )(input)
+    .upgrade()?;
+    let (_, hours) = trim_zeros_and_parse_u32(hours_text, "hours")?;
+    let (_, minutes) = trim_zeros_and_parse_u32(minutes_text, "minutes")?;
+    Ok((remaining, (hours, minutes)))
 }
 
 /// Matches a ':' followed by a two-digit second field. (`:44`)
-fn second(input: &str) -> IResult<&str, u32> {
-    map(
-        preceded(tag(":"), recognize(pair(one_of("012345"), digit))),
-        |seconds| trim_zeros_expect_u32(seconds, "seconds"),
-    )(input)
+fn second(input: &str) -> IonParseResult<u32> {
+    let (remaining, seconds) =
+        preceded(tag(":"), recognize(pair(one_of("012345"), digit)))(input).upgrade()?;
+    let (_, seconds) = trim_zeros_and_parse_u32(seconds, "seconds")?;
+    Ok((remaining, seconds))
 }
 
 /// Recognizes a decimal point followed by one or more digits.
-fn recognize_fractional_seconds(input: &str) -> IResult<&str, &str> {
-    preceded(tag("."), digit1)(input)
+fn recognize_fractional_seconds(input: &str) -> IonParseResult<&str> {
+    preceded(tag("."), digit1)(input).upgrade()
 }
 
 /// Matches a timezone offset (`Z`, `-00:00`, `-05:00`, or `+01:00`) and returns it as a number of
 /// minutes offset from UTC. Negative numbers indicate timezones west of UTC.
-fn timezone_offset(input: &str) -> IResult<&str, Option<i32>> {
+fn timezone_offset(input: &str) -> IonParseResult<Option<i32>> {
     alt((
         map(tag("Z"), |_| Some(0)),
         map(tag("-00:00"), |_| None),
         map(
             pair(one_of("-+"), timezone_offset_hours_minutes),
             |(sign, (hours, minutes))| {
-                let hours = trim_zeros_expect_i32(hours, "offset hours");
-                let minutes = trim_zeros_expect_i32(minutes, "offset minutes");
                 let offset_minutes = (hours * 60) + minutes;
                 if sign == '-' {
                     return Some(-offset_minutes);
@@ -270,13 +266,22 @@ fn timezone_offset(input: &str) -> IResult<&str, Option<i32>> {
 }
 
 /// Recognizes the hours and minutes portions of a timezone offset. (`12` and `35` in `12:35`)
-fn timezone_offset_hours_minutes(input: &str) -> IResult<&str, (&str, &str)> {
-    separated_pair(
-        // The parser does not restrict the range of hours/minutes allowed in the offset.
+fn timezone_offset_hours_minutes(input: &str) -> IonParseResult<(i32, i32)> {
+    let (remaining_input, (hours, minutes)) = separated_pair(
         recognize(pair(digit, digit)),
         tag(":"),
         recognize(pair(digit, digit)),
     )(input)
+    .upgrade()?;
+    let (_, hour_int) = trim_zeros_and_parse_i32(hours, "offset hours")?;
+    let (_, minute_int) = trim_zeros_and_parse_i32(minutes, "offset minutes")?;
+    if hour_int >= 24 {
+        return fatal_parse_error(hours, "timestamp hours were >= 24");
+    }
+    if minute_int >= 60 {
+        return fatal_parse_error(hours, "timestamp minutes were >= 60");
+    }
+    Ok((remaining_input, (hour_int, minute_int)))
 }
 
 #[cfg(test)]
@@ -447,3 +452,9 @@ mod reader_tests {
         Ok(())
     }
 }
+
+/// Matches the next input character if it is a base-10 digit. This wraps [char::is_digit] in the
+/// nom parsing function signature.
+pub(crate) fn digit(input: &str) -> IResult<&str, char> {
+    satisfy(|c| c.is_digit(10))(input)
+}
