@@ -1,7 +1,6 @@
 use std::fmt::Display;
 
-use nom::Err::Incomplete;
-use nom::IResult;
+use nom::Err::{Error, Failure, Incomplete};
 
 use crate::raw_reader::RawStreamItem;
 use crate::raw_symbol_token::RawSymbolToken;
@@ -10,15 +9,17 @@ use crate::result::{
 };
 use crate::stream_reader::StreamReader;
 use crate::text::parent_container::ParentContainer;
+use crate::text::parse_result::IonParseResult;
 use crate::text::parsers::containers::{
     list_delimiter, list_value_or_end, s_expression_delimiter, s_expression_value_or_end,
     struct_delimiter, struct_field_name_or_end, struct_field_value,
 };
-use crate::text::parsers::top_level::{ion_1_0_version_marker, top_level_value};
+use crate::text::parsers::top_level::{stream_item, RawTextStreamItem};
 use crate::text::text_buffer::TextBuffer;
 use crate::text::text_data_source::TextIonDataSource;
 use crate::text::text_value::{AnnotatedTextValue, TextValue};
 use crate::types::decimal::Decimal;
+use crate::types::integer::Integer;
 use crate::types::timestamp::Timestamp;
 use crate::IonType;
 
@@ -40,6 +41,21 @@ pub struct RawTextReader<T: TextIonDataSource> {
     parents: Vec<ParentContainer>,
 }
 
+/// Represents the final outcome of a [RawTextReader]'s attempt to parse the next value in the stream.
+///
+/// `IonParseResult<'a>` is not suitable for this purpose; its lifetime, `'a`, causes
+/// the result to hold a reference to the [RawTextReader]'s buffer even after parsing has finished,
+/// making it difficult to perform the necessary bookkeeping that follows finding the next value.
+///
+/// This type is intentionally limited in what it stores to avoid having a lifetime.
+#[derive(Eq, PartialEq, Debug)]
+pub(crate) enum RootParseResult<O> {
+    Ok(O),
+    Eof,
+    NoMatch,
+    Failure(String),
+}
+
 impl<T: TextIonDataSource> RawTextReader<T> {
     pub fn new(input: T) -> RawTextReader<T> {
         let text_source = input.to_text_ion_data_source();
@@ -62,11 +78,12 @@ impl<T: TextIonDataSource> RawTextReader<T> {
         // If the reader's current value is the beginning of a container and the user calls `next()`,
         // we need to skip the entire container. We can do this by stepping into and then out of
         // that container; `step_out()` has logic that will exhaust the remaining values.
-        let need_to_skip_container = self
-            .current_value
-            .as_ref()
-            .map(|v| v.value().ion_type().is_container())
-            .unwrap_or(false);
+        let need_to_skip_container = !self.is_null()
+            && self
+                .current_value
+                .as_ref()
+                .map(|v| v.value().ion_type().is_container())
+                .unwrap_or(false);
 
         if need_to_skip_container {
             self.step_in()?;
@@ -88,31 +105,10 @@ impl<T: TextIonDataSource> RawTextReader<T> {
                 return Ok(());
             }
 
-            // Otherwise, see if the next token in the stream is an Ion Version Marker.
-            if let Ok(Some(_)) = self.parse_next(ion_1_0_version_marker) {
-                // We found an IVM; we currently only support Ion 1.0.
-                self.current_ivm = Some((1, 0));
-                return Ok(());
-            }
-
-            // If it wasn't an IVM, it has to be a value.
-            let value = self.next_top_level_value();
-            match value {
-                Ok(None) => {
-                    // We hit EOF; make a note of it and clear the current value.
-                    self.is_eof = true;
-                    self.current_value = None;
-                }
-                Ok(Some(ref value)) => {
-                    // We read a value successfully; set it as our current value.
-                    // TODO: This currently clones the loaded value. This will not be necessary
-                    //       when `next()` returns an IonType instead of an AnnotatedTextValue.
-                    self.current_value = Some(value.clone());
-                }
-                _ => {}
-            };
-            return Ok(());
+            let next_stream_item = self.parse_next_nom(stream_item);
+            return self.process_stream_item(next_stream_item);
         }
+
         // Otherwise, the `parents` stack is not empty. We're inside a container.
 
         // The `ParentLevel` type is only a couple of stack-allocated bytes. It's very cheap to clone.
@@ -165,18 +161,55 @@ impl<T: TextIonDataSource> RawTextReader<T> {
         Ok(())
     }
 
-    /// Assumes that the reader is at the top level and attempts to parse the next value or IVM in
-    /// the stream.
-    fn next_top_level_value(&mut self) -> IonResult<Option<AnnotatedTextValue>> {
-        match self.parse_next(top_level_value) {
-            Ok(Some(value)) => Ok(Some(value)),
-            Ok(None) => {
+    fn process_stream_item(
+        &mut self,
+        read_result: RootParseResult<RawTextStreamItem>,
+    ) -> IonResult<()> {
+        match read_result {
+            RootParseResult::Ok(RawTextStreamItem::IonVersionMarker(1, 0)) => {
+                // We found an IVM; we currently only support Ion 1.0.
+                self.current_ivm = Some((1, 0));
+                Ok(())
+            }
+            RootParseResult::Ok(RawTextStreamItem::IonVersionMarker(major, minor)) => {
+                decoding_error(format!(
+                    "Unsupported Ion version: v{}.{}. Only 1.0 is supported.",
+                    major, minor
+                ))
+            }
+            RootParseResult::Ok(RawTextStreamItem::AnnotatedTextValue(value)) => {
+                // We read a value successfully; set it as our current value.
+                self.current_value = Some(value);
+                Ok(())
+            }
+            RootParseResult::Eof => {
                 // The top level is the only depth at which EOF is legal. If we encounter an EOF,
                 // double check that the buffer doesn't actually have a value in it. See the
                 // comments in [parse_value_at_eof] for a detailed explanation of this.
-                self.parse_value_at_eof()
+                let item = self.parse_value_at_eof();
+                if item == RootParseResult::Eof {
+                    // This is a genuine EOF; make a note of it and clear the current value.
+                    self.is_eof = true;
+                    self.current_value = None;
+                    return Ok(());
+                }
+                self.process_stream_item(item)
+            }
+            RootParseResult::NoMatch => {
+                // The parser didn't recognize the text in the input buffer.
+                // Return an error that contains the text we were attempting to parse.
+                let error_message = format!(
+                    "unrecognized input near line {}: '{}'",
+                    self.buffer.lines_loaded(),
+                    self.buffer.remaining_text(),
+                );
+                decoding_error(error_message)
+            }
+            RootParseResult::Failure(error_message) => {
+                // A fatal error occurred while reading the next value.
+                // This could be an I/O error, malformed utf-8 data, or an invalid value.
+                decoding_error(error_message)
             }
-            Err(e) => Err(e),
         }
     }
 
@@ -215,7 +248,7 @@ impl<T: TextIonDataSource> RawTextReader<T> {
     /// If the parser encounters an error, it will be returned as-is.
     fn parse_expected<P, O>(&mut self, entity_name: &str, parser: P) -> IonResult<O>
     where
-        P: Fn(&str) -> IResult<&str, O>,
+        P: Fn(&str) -> IonParseResult<O>,
     {
         match self.parse_next(parser) {
             Ok(Some(value)) => Ok(value),
@@ -235,25 +268,52 @@ impl<T: TextIonDataSource> RawTextReader<T> {
         }
     }
 
+    fn parse_next<P, O>(&mut self, parser: P) -> IonResult<Option<O>>
+    where
+        P: Fn(&str) -> IonParseResult<O>,
+    {
+        match self.parse_next_nom(parser) {
+            RootParseResult::Ok(item) => Ok(Some(item)),
+            RootParseResult::Eof => Ok(None),
+            RootParseResult::NoMatch => {
+                // Return an error that contains the text currently in the buffer (i.e. what we
+                // were attempting to parse.)
+                let error_message = format!(
+                    "unrecognized input near line {}: '{}'",
+                    self.buffer.lines_loaded(),
+                    self.buffer.remaining_text(),
+                );
+                decoding_error(error_message)
+            }
+            RootParseResult::Failure(error_message) => decoding_error(error_message),
+        }
+    }
+
     /// Attempts to parse the next entity from the stream using the provided parser.
     /// If there isn't enough data in the buffer for the parser to match its input conclusively,
     /// more data will be loaded into the buffer and the parser will be called again.
     /// If EOF is encountered, returns `Ok(None)`.
-    fn parse_next<P, O>(&mut self, parser: P) -> IonResult<Option<O>>
+    fn parse_next_nom<P, O>(&mut self, parser: P) -> RootParseResult<O>
     where
-        P: Fn(&str) -> IResult<&str, O>,
+        P: Fn(&str) -> IonParseResult<O>,
     {
-        if self.is_eof {
-            return Ok(None);
+        let RawTextReader {
+            ref mut is_eof,
+            ref mut buffer,
+            ref mut bytes_read,
+            ..
+        } = *self;
+
+        if *is_eof {
+            return RootParseResult::Eof;
         }
 
-        let value = 'parse: loop {
+        loop {
             // Note the number of bytes currently in the text buffer
-            let length_before_parse = self.buffer.remaining_text().len();
-            let input_text = self.buffer.remaining_text();
+            let length_before_parse = buffer.remaining_text().len();
             // Invoke the top_level_value() parser; this will attempt to recognize the next value
             // in the stream and return a &str slice containing the remaining, not-yet-parsed text.
-            match parser(input_text) {
+            match parser(buffer.remaining_text()) {
                 // If `top_level_value` returns 'Incomplete', there wasn't enough text in the buffer
                 // to match the next value. No syntax errors have been encountered (yet?), but we
                 // need to load more text into the buffer before we try to parse it again.
@@ -262,15 +322,25 @@ impl<T: TextIonDataSource> RawTextReader<T> {
                     // TODO: Currently this loads a single line at a time for easier testing.
                     //       We may wish to bump it to a higher number of lines at a time (8?)
                     //       for efficiency once we're confident in the correctness.
-                    if self.buffer.load_next_line()? == 0 {
-                        // If load_next_line() returns Ok(0), we've reached the end of our input.
-                        self.is_eof = true;
-                        // The buffer had an `Incomplete` value in it; now that we know we're at EOF,
-                        // we can determine whether the buffer's contents should actually be
-                        // considered complete.
-                        return Ok(None);
+                    match buffer.load_next_line() {
+                        Ok(0) => {
+                            // If load_next_line() returns Ok(0), we've reached the end of our input.
+                            *is_eof = true;
+                            // The buffer had an `Incomplete` value in it; now that we know we're at EOF,
+                            // we can determine whether the buffer's contents should actually be
+                            // considered complete.
+                            return RootParseResult::Eof;
+                        }
+                        Ok(_bytes_loaded) => {
+                            // Retry the parser on the extended buffer in the next loop iteration
+                            continue;
+                        }
+                        Err(e) => {
+                            let error_message =
+                                format!("I/O error, could not read more data: {}", e);
+                            return RootParseResult::Failure(error_message);
+                        }
                     }
-                    continue;
                 }
                 Ok((remaining_text, value)) => {
                     // Our parser successfully matched a value.
@@ -279,28 +349,22 @@ impl<T: TextIonDataSource> RawTextReader<T> {
                     // The difference in length tells us how many bytes were part of the
                     // text representation of the value that we found.
                     let bytes_consumed = length_before_parse - length_after_parse;
-                    // Discard `bytes_consumed` bytes from the TextBuffer.
-                    self.buffer.consume(bytes_consumed);
-                    self.bytes_read += bytes_consumed;
-                    // Break out of the read/parse loop, returning the value that we matched.
-                    break 'parse value;
+                    buffer.consume(bytes_consumed);
+                    *bytes_read += bytes_consumed;
+                    return RootParseResult::Ok(value);
                 }
-                Err(e) => {
-                    // Return an error that contains the text currently in the buffer (i.e. what we
-                    // were attempting to parse with `top_level_value`.)
-                    // TODO: We probably don't want to surface the nom error (`e`) directly, but it's
-                    //       useful for debugging.
-                    return decoding_error(format!(
-                        "Parsing error occurred near line {}: '{}': '{}'",
-                        self.buffer.lines_loaded(),
-                        self.buffer.remaining_text(),
-                        e
-                    ));
+                Err(Error(_e)) => return RootParseResult::<O>::NoMatch,
+                Err(Failure(e)) => {
+                    let error_message = format!(
+                        "unrecognized input near line {}: {}: '{}'",
+                        buffer.lines_loaded(),
+                        e.description().unwrap_or("<no description>"),
+                        buffer.remaining_text(),
+                    );
+                    return RootParseResult::Failure(error_message);
                 }
             };
-        };
-
-        Ok(Some(value))
+        }
     }
 
     // Parses the contents of the text buffer again with the knowledge that we're at the end of the
@@ -309,7 +373,7 @@ impl<T: TextIonDataSource> RawTextReader<T> {
     // https://github.com/amzn/ion-rust/issues/318
     // This method should only be called when the reader is at the top level. An EOF at any other
     // depth is an error.
-    fn parse_value_at_eof(&mut self) -> IonResult<Option<AnnotatedTextValue>> {
+    fn parse_value_at_eof(&mut self) -> RootParseResult<RawTextStreamItem> {
         // An arbitrary, cheap-to-parse Ion value that we append to the buffer when its contents at
         // EOF are ambiguous.
         const SENTINEL_ION_TEXT: &str = "\n0\n";
@@ -331,34 +395,43 @@ impl<T: TextIonDataSource> RawTextReader<T> {
         //   there aren't any more long-form string segments in the sequence.
         //
         // Attempt to parse the updated buffer.
-        let value = match top_level_value(self.buffer.remaining_text()) {
-            Ok(("\n", value))
-                if value.annotations().is_empty() && *value.value() == TextValue::Integer(0) =>
+        let value = match stream_item(self.buffer.remaining_text()) {
+            Ok(("\n", RawTextStreamItem::AnnotatedTextValue(value)))
+                if value.annotations().is_empty()
+                    && *value.value() == TextValue::Integer(Integer::I64(0)) =>
             {
                 // We found the unannotated zero that we appended to the end of the buffer.
                 // The "\n" in this pattern is the unparsed text left in the buffer,
                 // which indicates that our 0 was parsed.
-                Ok(None)
+                RootParseResult::Eof
             }
             Ok((_remaining_text, value)) => {
                 // We found something else. The zero is still in the buffer; we can leave it there.
                 // The reader's `is_eof` flag has been set, so the text buffer will never be used
                 // again. Return the value we found.
-                Ok(Some(value))
+                RootParseResult::Ok(value)
             }
             Err(Incomplete(_needed)) => {
-                decoding_error(format!(
+                RootParseResult::Failure(format!(
                     "Unexpected end of input on line {}: '{}'",
                     self.buffer.lines_loaded(),
                     &self.buffer.remaining_text()[..original_length] // Don't show the extra `\n0\n`
                 ))
             }
-            Err(e) => {
-                decoding_error(format!(
-                    "Parsing error occurred near line {}: '{}': '{}'",
+            Err(Error(ion_parse_error)) => {
+                RootParseResult::Failure(format!(
+                    "Parsing error occurred near line {}: '{}': '{:?}'",
                     self.buffer.lines_loaded(),
                     &self.buffer.remaining_text()[..original_length], // Don't show the extra `\n0\n`
-                    e
+                    ion_parse_error
+                ))
+            }
+            Err(Failure(ion_parse_error)) => {
+                RootParseResult::Failure(format!(
+                    "A fatal error occurred while reading near line {}: '{}': '{:?}'",
+                    self.buffer.lines_loaded(),
+                    &self.buffer.remaining_text()[..original_length], // Don't show the extra `\n0\n`
+                    ion_parse_error
                 ))
             }
         };
@@ -501,9 +574,19 @@ impl<T: TextIonDataSource> StreamReader for RawTextReader<T> {
         }
     }
 
+    fn read_integer(&mut self) -> IonResult<Integer> {
+        match self.current_value.as_ref().map(|current| current.value()) {
+            Some(TextValue::Integer(value)) => Ok(value.clone()),
+            _ => Err(self.expected("int value")),
+        }
+    }
+
     fn read_i64(&mut self) -> IonResult<i64> {
         match self.current_value.as_ref().map(|current| current.value()) {
-            Some(TextValue::Integer(value)) => Ok(*value),
+            Some(TextValue::Integer(Integer::I64(value))) => Ok(*value),
+            Some(TextValue::Integer(Integer::BigInt(value))) => {
+                decoding_error(format!("Integer {} is too large to fit in an i64.", value))
+            }
             _ => Err(self.expected("int value")),
         }
     }
@@ -674,6 +757,7 @@ impl<T: TextIonDataSource> StreamReader for RawTextReader<T> {
 mod reader_tests {
     use rstest::*;
 
+    use super::*;
     use crate::raw_reader::RawStreamItem;
     use crate::raw_symbol_token::{local_sid_token, text_token, RawSymbolToken};
     use crate::result::IonResult;
@@ -803,7 +887,7 @@ mod reader_tests {
     #[case(" null.string ", TextValue::Null(IonType::String))]
     #[case(" true ", TextValue::Boolean(true))]
     #[case(" false ", TextValue::Boolean(false))]
-    #[case(" 738 ", TextValue::Integer(738))]
+    #[case(" 738 ", TextValue::Integer(Integer::I64(738)))]
     #[case(" 2.5e0 ", TextValue::Float(2.5))]
     #[case(" 2.5 ", TextValue::Decimal(Decimal::new(25, -1)))]
     #[case(" 2007-07-12T ", TextValue::Timestamp(Timestamp::with_ymd(2007, 7, 12).build().unwrap()))]
@@ -1044,4 +1128,58 @@ mod reader_tests {
 
         Ok(())
     }
+
+    #[test]
+    fn structs_trailing_comma() -> IonResult<()> {
+        let pretty_ion = br#"
+            // Structs with last field with/without trailing comma
+            (
+                {a:1, b:2,}     // with trailing comma
+                {a:1, b:2 }     // without trailing comma
+            )
+        "#;
+        let mut reader = RawTextReader::new(&pretty_ion[..]);
+        assert_eq!(reader.next()?, RawStreamItem::Value(IonType::SExpression));
+        reader.step_in()?;
+        assert_eq!(reader.next()?, RawStreamItem::Value(IonType::Struct));
+
+        reader.step_in()?;
+        assert_eq!(reader.next()?, RawStreamItem::Value(IonType::Integer));
+        assert_eq!(reader.field_name()?, RawSymbolToken::Text("a".to_string()));
+        assert_eq!(reader.read_i64()?, 1);
+        assert_eq!(reader.next()?, RawStreamItem::Value(IonType::Integer));
+        assert_eq!(reader.field_name()?, RawSymbolToken::Text("b".to_string()));
+        assert_eq!(reader.read_i64()?, 2);
+        reader.step_out()?;
+
+        assert_eq!(reader.next()?, RawStreamItem::Value(IonType::Struct));
+        reader.step_out()?;
+        Ok(())
+    }
+
+    #[test]
+    fn annotation_false() -> IonResult<()> {
+        // The reader will reject the unquoted boolean keyword 'false' when used as an annotation
+        let pretty_ion = br#"
+            false::23
+        "#;
+        let mut reader = RawTextReader::new(&pretty_ion[..]);
+        let result = reader.next();
+        println!("{:?}", result);
+        assert!(result.is_err());
+        Ok(())
+    }
+
+    #[test]
+    fn annotation_nan() -> IonResult<()> {
+        // The reader will reject the unquoted float keyword 'nan' when used as an annotation
+        let pretty_ion = br#"
+            nan::23
+        "#;
+        let mut reader = RawTextReader::new(&pretty_ion[..]);
+        let result = reader.next();
+        println!("{:?}", result);
+        assert!(result.is_err());
+        Ok(())
+    }
 }
