@@ -354,6 +354,12 @@ impl TimestampBuilder {
     where
         D: Datelike + Timelike + Debug,
     {
+        if self.year == 0 || self.year > 9999 {
+            return illegal_operation(format!(
+                "Timestamp year '{}' out of range (1-9999)",
+                self.year
+            ));
+        }
         datetime = datetime.with_year(self.year as i32).ok_or_else(|| {
             illegal_operation_raw(format!("specified year ('{}') is invalid", self.year))
         })?;
@@ -488,6 +494,18 @@ impl TimestampBuilder {
 
         // Copy the fractional seconds from the builder to the Timestamp.
         if self.precision == Precision::FractionalSeconds {
+            if let Some(Mantissa::Arbitrary(ref decimal)) = self.fractional_seconds {
+                if decimal.is_less_than_zero() {
+                    return illegal_operation(
+                        "cannot create a timestamp with negative fractional seconds",
+                    );
+                }
+                if decimal.is_greater_than_or_equal_to_one() {
+                    return illegal_operation(
+                        "cannot create a timestamp with a fractional seconds >= 1.0",
+                    );
+                }
+            }
             timestamp.fractional_seconds = self.fractional_seconds;
         }
         Ok(timestamp)
@@ -614,7 +632,7 @@ impl SecondSetter {
         self.into_builder().build()
     }
 
-    /// Like [build_at_offset], but the fields provided for each time unit are understood
+    /// Like [Self::build_at_offset], but the fields provided for each time unit are understood
     /// to be in UTC rather than in the local time of the specified offset.
     pub fn build_utc_fields_at_offset(mut self, offset_minutes: i32) -> IonResult<Timestamp> {
         self.builder.fields_are_utc = true;
@@ -686,7 +704,7 @@ impl FractionalSecondSetter {
         self.into_builder().build()
     }
 
-    /// Like [build_at_offset], but the fields provided for each time unit are understood
+    /// Like [Self::build_at_offset], but the fields provided for each time unit are understood
     /// to be in UTC rather than in the local time of the specified offset.
     pub fn build_utc_fields_at_offset(mut self, offset_minutes: i32) -> IonResult<Timestamp> {
         self.builder.fields_are_utc = true;
