@@ -5,8 +5,10 @@
 //! This API is simpler to manage with respect to borrowing lifetimes, but requires full
 //! ownership of data to do so.
 
-use super::{AnyInt, Element, ImportSource, Sequence, Struct, SymbolToken};
+use super::{Element, ImportSource, Sequence, Struct, SymbolToken};
+use crate::ion_eq::IonEq;
 use crate::types::decimal::Decimal;
+use crate::types::integer::Integer;
 use crate::types::timestamp::Timestamp;
 use crate::types::SymbolId;
 use crate::value::Builder;
@@ -172,11 +174,11 @@ impl Builder for OwnedElement {
     }
 
     fn new_i64(int: i64) -> Self::Element {
-        OwnedValue::Integer(AnyInt::I64(int)).into()
+        OwnedValue::Integer(Integer::I64(int)).into()
     }
 
     fn new_big_int(big_int: BigInt) -> Self::Element {
-        OwnedValue::Integer(AnyInt::BigInt(big_int)).into()
+        OwnedValue::Integer(Integer::BigInt(big_int)).into()
     }
 
     fn new_decimal(decimal: Decimal) -> Self::Element {
@@ -282,7 +284,7 @@ impl OwnedStruct {
         self.text_fields.iter().all(|(key, value)| {
             value
                 .iter()
-                .all(|(_my_s, my_v)| other.get_all(key).any(|other_v| my_v == other_v))
+                .all(|(_my_s, my_v)| other.get_all(key).any(|other_v| my_v.ion_eq(other_v)))
                 && value.len() == other.get_all(key).count()
         })
     }
@@ -293,7 +295,7 @@ impl OwnedStruct {
             other
                 .no_text_fields
                 .iter()
-                .any(|(other_k, other_v)| my_k == other_k && my_v == other_v)
+                .any(|(other_k, other_v)| my_k == other_k && my_v.ion_eq(other_v))
         })
     }
 }
@@ -388,11 +390,51 @@ impl PartialEq for OwnedStruct {
 
 impl Eq for OwnedStruct {}
 
+impl IonEq for OwnedSequence {
+    fn ion_eq(&self, other: &Self) -> bool {
+        self.children.ion_eq(&other.children)
+    }
+}
+
+impl IonEq for OwnedValue {
+    fn ion_eq(&self, other: &Self) -> bool {
+        use OwnedValue::*;
+        match (self, other) {
+            (Float(f1), Float(f2)) => return f1.ion_eq(f2),
+            (List(seq1), List(seq2)) => return seq1.ion_eq(seq2),
+            (SExpression(seq1), SExpression(seq2)) => return seq1.ion_eq(seq2),
+            _ => {}
+        };
+        // For any other case, fall back to vanilla equality
+        self == other
+    }
+}
+
+impl IonEq for OwnedElement {
+    fn ion_eq(&self, other: &Self) -> bool {
+        self.annotations == other.annotations && self.value.ion_eq(&other.value)
+    }
+}
+
+impl IonEq for Vec<OwnedElement> {
+    fn ion_eq(&self, other: &Self) -> bool {
+        if self.len() != other.len() {
+            return false;
+        }
+        for (v1, v2) in self.iter().zip(other.iter()) {
+            if !v1.ion_eq(v2) {
+                return false;
+            }
+        }
+        true
+    }
+}
+
 /// Variants for all owned version _values_ within an [`Element`].
 #[derive(Debug, Clone, PartialEq)]
 pub enum OwnedValue {
     Null(IonType),
-    Integer(AnyInt),
+    Integer(Integer),
     Float(f64),
     Decimal(Decimal),
     Timestamp(Timestamp),
@@ -442,13 +484,13 @@ impl From<IonType> for OwnedElement {
 
 impl From<i64> for OwnedElement {
     fn from(i64_val: i64) -> Self {
-        OwnedValue::Integer(AnyInt::I64(i64_val)).into()
+        OwnedValue::Integer(Integer::I64(i64_val)).into()
     }
 }
 
 impl From<BigInt> for OwnedElement {
     fn from(big_int_val: BigInt) -> Self {
-        OwnedValue::Integer(AnyInt::BigInt(big_int_val)).into()
+        OwnedValue::Integer(Integer::BigInt(big_int_val)).into()
     }
 }
 
@@ -532,7 +574,7 @@ impl Element for OwnedElement {
         matches!(&self.value, OwnedValue::Null(_))
     }
 
-    fn as_any_int(&self) -> Option<&AnyInt> {
+    fn as_integer(&self) -> Option<&Integer> {
         match &self.value {
             OwnedValue::Integer(i) => Some(i),
             _ => None,
